/***************************************************************************
 *   Copyright (C) 2005 by Rajko Albrecht   *
 *   ral@alwins-world.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "filelistviewitem.h"
#include "kdesvnfilelist.h"
#include "helpers/sub2qt.h"
#include "svncpp/status.hpp"
#include "svncpp/revision.hpp"
#include "svncpp/exception.hpp"
#include "svncpp/url.hpp"

#include <qfileinfo.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kfileitem.h>
#include <kdebug.h>

#include <svn_time.h>

const int FileListViewItem::COL_ICON = 0;
const int FileListViewItem::COL_NAME = 0;
const int FileListViewItem::COL_STATUS = 1;
const int FileListViewItem::COL_LAST_REV = 2;
const int FileListViewItem::COL_LAST_AUTHOR = 3;
const int FileListViewItem::COL_LAST_DATE = 4;
const int FileListViewItem::COL_IS_LOCKED = 5;

//const int FileListViewItem::COL_CURRENT_REV = 5;

FileListViewItem::FileListViewItem(kdesvnfilelist*_parent,const svn::Status&_stat)
 : KListViewItem(_parent),
 sortChar(0),
 m_Ksvnfilelist(_parent),
 m_Stat(_stat)
{
    init();
}

FileListViewItem::FileListViewItem(kdesvnfilelist*_parent,FileListViewItem*_parentItem,const svn::Status&_stat)
    : KListViewItem(_parentItem),
    sortChar(0),
    m_Ksvnfilelist(_parent),
    m_Stat(_stat)
{
    init();
}

void FileListViewItem::init()
{
    m_fullName = QString::fromLocal8Bit(m_Stat.path());
    while (m_fullName.endsWith("/")) {
        /* dir name possible */
        m_fullName.truncate(m_fullName.length()-1);
    }
    int p = m_fullName.findRev("/");
    if (p>-1) {
        ++p;
        m_shortName = m_fullName.right(m_fullName.length()-p);
    } else {
        m_shortName = m_fullName;
    }

    setText(COL_NAME,m_shortName);

    sortChar = isDir()?1:3;
    if (m_shortName[0]=='.') --sortChar;
    update();
}

bool FileListViewItem::isDir()const
{
//    if (QString::compare(stat.entry().url(),stat.path())==0) {
    if (m_Stat.entry().isValid()) {
        return m_Stat.entry().kind()==svn_node_dir;
    }
    /* must be a local file */
    QFileInfo f(m_Stat.path());
    return f.isDir();
}

FileListViewItem::~FileListViewItem()
{
}

void FileListViewItem::update(KFileItem*_item)
{
    setText(COL_NAME,_item->name());
    setPixmap(COL_ICON,_item->pixmap(16,0));
    sortChar = S_ISDIR( _item->mode() ) ? 1 : 3;
    if ( _item->name()[0] == '.' )
        --sortChar;
    try {
      m_Stat = m_Ksvnfilelist->svnclient()->singleStatus(_item->url().path());
    } catch (svn::ClientException e) {
        setText(COL_STATUS,e.message());
        return;
    }
    update();
}

void FileListViewItem::refreshMe()
{
    m_Stat = svn::Status();
    try {
        m_Stat = m_Ksvnfilelist->svnclient()->singleStatus(fullName().local8Bit());
    } catch (svn::ClientException e) {
        setText(COL_STATUS,e.message());
        return;
    }
    update();
}

bool FileListViewItem::isRealVersioned()const
{
    return m_Stat.isRealVersioned();
}

void FileListViewItem::checkNewer()
{
#if 0
    if (!svn::Url::isValid (fullName().local8Bit()) && m_Stat.isRealVersioned()) {
        svn::Status tmpStat = m_Ksvnfilelist->svnclient()->singleStatus(fullName().local8Bit(),true);
        if (tmpStat.reposTextStatus()!=svn_wc_status_none||tmpStat.reposPropStatus()!=svn_wc_status_none) {
            setText(COL_STATUS,i18n("Needs update"));
        }
    }
#endif
}

void FileListViewItem::refreshStatus(bool childs,QPtrList<FileListViewItem>*exclude,bool depsonly)
{
    FileListViewItem*it;

    if (!depsonly) {
        refreshMe();
    }
    if (!isValid()) {
        return;
    }
    it = static_cast<FileListViewItem*>(parent());;
    if (!childs) {
        if (it && (!exclude || exclude->find(it)==-1)) {
            it->refreshStatus(false,exclude);
        }
    } else if (firstChild()){
        it = static_cast<FileListViewItem*>(firstChild());
        while (it) {
            if (!exclude || exclude->find(it)==-1) {
                it->refreshStatus(true,exclude);
            }
            it = static_cast<FileListViewItem*>(it->nextSibling());
        }
    }
    repaint();
}

void FileListViewItem::makePixmap()
{
    QPixmap p;
    /* yes - different way to "isDir" above 'cause here we try to use the
       mime-features of KDE on ALL not just unversioned entries.
     */
    if (QString::compare(m_Stat.entry().url(),m_Stat.path())==0) {
        /* remote access */
        if (isDir()) {
            p = KGlobal::iconLoader()->loadIcon("folder",KIcon::Desktop,16);
        } else {
            p = KGlobal::iconLoader()->loadIcon("unknown",KIcon::Desktop,16);
            //p = KMimeType::pixmapForURL(KURL(stat.entry().url()),0,KIcon::Desktop,16);
        }
    } else {
        p = KMimeType::pixmapForURL(m_Stat.path(),0,KIcon::Desktop,16);
    }
    setPixmap(COL_ICON,p);
}

bool FileListViewItem::isVersioned()
{
    return m_Stat.isVersioned();
}

bool FileListViewItem::isValid()
{
    if (isVersioned()) {
        return true;
    }
    /* must be a local file */
    //kdDebug()<< "Pfad = " << m_Stat.path()<<endl;

    QFileInfo f(m_Stat.path());
    //kdDebug()<< "Behaupte das es existiert: "<<f.exists()<<endl;
    return f.exists();
}

bool FileListViewItem::isParent(QListViewItem*which)
{
    if (!which) return false;
    QListViewItem*item = this;
    while ( (item=item->parent())) {
        if (item==which) {
            return true;
        }
    }
    return false;
}

bool FileListViewItem::isIgnored()const
{
    return m_Stat.textStatus()==svn_wc_status_ignored;
}

void FileListViewItem::update()
{
    makePixmap();
    if (!m_Stat.isVersioned()) {
        setText(COL_STATUS,i18n("Not versioned"));
        return;
    }
    QString info_text = "";
    switch(m_Stat.textStatus ()) {
    case svn_wc_status_modified:
        info_text = i18n("Locally modified");
        break;
    case svn_wc_status_added:
        info_text = i18n("Locally added");
        break;
    case svn_wc_status_missing:
        info_text = i18n("Missing");
        break;
    case svn_wc_status_deleted:
        info_text = i18n("Deleted");
        break;
    case svn_wc_status_replaced:
        info_text = i18n("Replaced");
        break;
    case svn_wc_status_ignored:
        info_text = i18n("Ignored");
        break;
    case svn_wc_status_external:
        info_text=i18n("External");
        break;
    case svn_wc_status_conflicted:
        info_text=i18n("Conflict");
        break;
    case svn_wc_status_merged:
        info_text=i18n("Merged");
        break;
    case svn_wc_status_incomplete:
        info_text=i18n("Incomplete");
        break;
    default:
        break;
    }
    if (info_text.isEmpty()) {
        switch (m_Stat.propStatus ()) {
        case svn_wc_status_modified:
            info_text = i18n("Property modified");
            break;
        default:
            break;
        }
    }
    setText(COL_STATUS,info_text);
    setText(COL_LAST_AUTHOR,m_Stat.entry().cmtAuthor());
    fullDate = helpers::sub2qt::apr_time2qt(m_Stat.entry().cmtDate());
    setText(COL_LAST_DATE,fullDate.toString());
    setText(COL_LAST_REV,QString("%1").arg(m_Stat.entry().cmtRev()));
#if 1
    if (m_Stat.entry().lockEntry().Locked()) {
        setPixmap(COL_IS_LOCKED,KGlobal::iconLoader()->loadIcon("lock",KIcon::Desktop,16));
    } else {
        setPixmap(COL_IS_LOCKED,QPixmap());
    }
    setText(COL_IS_LOCKED,QString::fromLocal8Bit(m_Stat.entry().lockEntry().Owner().c_str()));
#endif
    checkNewer();
//    setText(COL_CURRENT_REV,QString("%1").arg(stat.entry().revision()));
}

int FileListViewItem::compare( QListViewItem* item, int col, bool ascending ) const
{
    FileListViewItem* k = static_cast<FileListViewItem*>( item );
    if ( sortChar != k->sortChar ) {
        // Dirs are always first, even when sorting in descending order
        return !ascending ? k->sortChar - sortChar : sortChar - k->sortChar;
    }
    if (col==COL_LAST_DATE) {
        return fullDate.secsTo(k->fullDate);
    }
    if (col==COL_LAST_REV) {
        return k->m_Stat.entry().cmtRev()-m_Stat.entry().cmtRev();
    }
    return text(col).localeAwareCompare(k->text(col));
}

void FileListViewItem::removeChilds()
{
    QListViewItem*temp;
    while ((temp=firstChild())) {
        delete temp;
    }
}

void FileListViewItem::updateStatus(const svn::Status&s)
{
    m_Stat = s;
    init();
}


/*!
    \fn FileListViewItem::getParentDir()const
 */
 QString FileListViewItem::getParentDir()const
{
    FileListViewItem*temp = static_cast<FileListViewItem*>(parent());
    if (!temp) return QString::null;
    return temp->fullName();
}
